@@ -21,12 +21,15 @@ from tagging.tests.models import Perch
 from tagging.tests.models import Parrot
 from tagging.tests.models import FormTest
 from tagging.tests.models import FormTestNull
+from tagging.tests.models import FormMultipleFieldTest
 from tagging.utils import LINEAR
+from tagging.utils import LOGARITHMIC
 from tagging.utils import get_tag
 from tagging.utils import get_tag_list
 from tagging.utils import calculate_cloud
 from tagging.utils import parse_tag_input
 from tagging.utils import edit_string_for_tags
+from tagging.utils import _calculate_tag_weight
 
 #############
 # Utilities #
@@ -249,6 +252,22 @@ class TestCalculateCloud(TestCase):
             raise self.failureException(
                 'a ValueError exception was supposed to be raised!')
 
+    def test_calculate_tag_weight(self):
+        self.assertEqual(
+            _calculate_tag_weight(10, 20, LINEAR),
+            10)
+        self.assertEqual(
+            _calculate_tag_weight(10, 20, LOGARITHMIC),
+            15.37243573680482)
+
+    def test_calculate_tag_weight_invalid_size(self):
+        self.assertEqual(
+            _calculate_tag_weight(10, 10, LOGARITHMIC),
+            10.0)
+        self.assertEqual(
+            _calculate_tag_weight(26, 26, LOGARITHMIC),
+            26.0)
+
 ###########
 # Tagging #
 ###########
@@ -442,6 +461,39 @@ class TestModelTagField(TestCase):
         f1 = FormTestNull()
         self.assertEqual(f1.tags, '')
 
+    def test_fix_update_tag_field_deferred(self):
+        """
+        Bug introduced in Django 1.10
+        the TagField is considered "deferred" on Django 1.10
+        because instance.__dict__ is not populated by the TagField
+        instance, so it's excluded when updating a model instance.
+
+        Note: this does not append if you only have one TagField
+        in your model...
+        """
+        f1 = FormMultipleFieldTest.objects.create(tagging_field='one two')
+        self.assertEqual(f1.tagging_field, 'one two')
+        tags = Tag.objects.get_for_object(f1)
+        self.assertEqual(len(tags), 2)
+        test1_tag = get_tag('one')
+        test2_tag = get_tag('two')
+        self.assertTrue(test1_tag in tags)
+        self.assertTrue(test2_tag in tags)
+
+        f1.tagging_field = f1.tagging_field + ' three'
+        f1.save()
+        self.assertEqual(f1.tagging_field, 'one two three')
+        tags = Tag.objects.get_for_object(f1)
+        self.assertEqual(len(tags), 3)
+        test3_tag = get_tag('three')
+        self.assertTrue(test3_tag in tags)
+
+        f1again = FormMultipleFieldTest.objects.get(pk=f1.pk)
+        self.assertEqual(f1again.tagging_field, 'one two three')
+
+        tags = Tag.objects.get_for_object(f1again)
+        self.assertEqual(len(tags), 3)
+
 
 class TestSettings(TestCase):
     def setUp(self):
@@ -1063,6 +1115,7 @@ class TestTagFieldInForms(TestCase):
         spaces = Tag.objects.create(name='spa ces')
         comma = Tag.objects.create(name='com,ma')
         self.assertEqual(edit_string_for_tags([plain]), 'plain')
+        self.assertEqual(edit_string_for_tags([spaces]), '"spa ces"')
         self.assertEqual(edit_string_for_tags([plain, spaces]),
                          'plain, spa ces')
         self.assertEqual(edit_string_for_tags([plain, spaces, comma]),
@@ -1122,9 +1175,14 @@ class TestTagAdminForm(TestCase):
 
 @override_settings(
     ROOT_URLCONF='tagging.tests.urls',
-    TEMPLATE_LOADERS=(
-        'tagging.tests.utils.VoidLoader',
-    ),
+    TEMPLATES=[
+        {
+            'BACKEND': 'django.template.backends.django.DjangoTemplates',
+            'OPTIONS': {
+                'loaders': ('tagging.tests.utils.VoidLoader',)
+            }
+        }
+    ]
 )
 class TestTaggedObjectList(TestCase):
 
